DoItAll = DoItAll or {}
local DoItAllSlots = DoItAll.Slots:New(DoItAll.ItemFilter:New(true))
local extractFunction, container
local extractionActive = false

function DoItAll.IsShowingExtraction()
	return not ZO_EnchantingTopLevelExtractionSlotContainer:IsHidden()
end

function DoItAll.IsShowingDeconstruction()
	return not ZO_SmithingTopLevelDeconstructionPanelSlotContainer:IsHidden()
end

function DoItAll.IsShowingRefinement()
	return not ZO_SmithingTopLevelRefinementPanelSlotContainer:IsHidden()
end

local function GetExtractionContainerAndFunction()
  if DoItAll.IsShowingExtraction() then
    return ZO_EnchantingTopLevelInventoryBackpack, ExtractEnchantingItem
  elseif DoItAll.IsShowingDeconstruction() then
    return ZO_SmithingTopLevelDeconstructionPanelInventoryBackpack, ExtractOrRefineSmithingItem
  elseif DoItAll.IsShowingRefinement() then
    return ZO_SmithingTopLevelRefinementPanelInventoryBackpack, ExtractOrRefineSmithingItem
  end
end

local function GetNextSlotToExtract()
  if not DoItAllSlots:Fill(container, 1) then
	DoItAllSlots:ClearNotAllowed()
  	return nil
  end
  return DoItAllSlots:Next()
end

local function ExtractionFinished()
	extractionActive = false
	EVENT_MANAGER:UnregisterForEvent("DoItAllExtraction", EVENT_CRAFT_COMPLETED)
	EVENT_MANAGER:UnregisterForEvent("DoItAllExtraction", EVENT_END_CRAFTING_STATION_INTERACT)
end

local function ExtractNext(firstExtract)
	local delayMs = 0
	firstExtract = firstExtract or false

--d("[DoItAll]ExtractNext("..tostring(firstExtract)..")")

	--get the next slot to extract
	local doitall_slot = GetNextSlotToExtract()
	if not doitall_slot then
		--No slot left -> Finish here
		ExtractionFinished()
	else
		local extractable = true
		--Are we inside refinement?
	    --Is the current slot's stackCount < 10 (no refinement is possible then)?
		if DoItAll.IsShowingRefinement() and doitall_slot.stackCount < 10 then
	       	extractable = false
		end
		if extractable then
			if not firstExtract then
				--Get the MS for the delay between extractions
				delayMs = DoItAll.Settings.GetExtractDelay()
            else
            	delayMs = 0
            end
--d("[DoItAll]ExtractNext delay: " .. tostring(delayMs))
            if delayMs > 0 then
				--Call the extraction function with a delay
				zo_callLater(function() extractFunction(doitall_slot.bagId, doitall_slot.slotIndex) end, delayMs)
            else
				extractFunction(doitall_slot.bagId, doitall_slot.slotIndex)
            end
		else
			--Disallow this item to be tried to extracted again
			DoItAllSlots:AddToNotAllowed(doitall_slot.bagId, doitall_slot.slotIndex)
            --Go on with next slot as the event EVENT_CRAFT_COMPLETED won't be called
			ExtractNext(false)
	    end
	end
end

--Event handler for EVENT_END_CRAFTING_STATION_INTERACT
local function OnCraftingEnd(eventCode)
	if extractionActive then
		d("[DoItAll] Extraction was aborted!")
		ExtractionFinished()
	end
end

local function StartExtraction()
	extractionActive = true

	--Register event to check if extraction has been aborted -> Avoid automatic extraction next time an item has been extracted
	EVENT_MANAGER:RegisterForEvent("DoItAllExtraction", EVENT_END_CRAFTING_STATION_INTERACT, OnCraftingEnd)
	EVENT_MANAGER:RegisterForEvent("DoItAllExtraction", EVENT_CRAFT_COMPLETED, function() ExtractNext(false) end)
	ExtractNext(true)
end

function DoItAll.ExtractAll()
  container, extractFunction = GetExtractionContainerAndFunction()
  if container == nil then return end
	--Clear/Initialize the not allowed slot entries
	DoItAllSlots:ClearNotAllowed()
	--Start the extraction
	StartExtraction()
end
